from django.contrib.contenttypes.models import ContentType
from django.forms import ModelChoiceField, ModelForm
from models import FieldSanitizer

class ContentTypeChoiceFieldWithAppLabel(ModelChoiceField):
    """
    A ModelChoiceField for ContentTypes that displays the 
    app_label in addition to the name. 
    """
    def label_from_instance(self, obj):
        return "%s/%s" % (obj.app_label, obj.name)

class FieldSanitizerAdminForm(ModelForm):
    content_type = ContentTypeChoiceFieldWithAppLabel(ContentType.objects.all().order_by('app_label', 'model'))
    class Meta:
        model = FieldSanitizer



"""
Management utility to clean Models that are registered with a FieldSanitizer.
"""

from django.core.management.base import NoArgsCommand
from janitor.models import FieldSanitizer, _clean_class_objects

class Command(NoArgsCommand):

    help = "Cleans all Models that have a related FieldSanitizer"

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)

        # BaseCommand doesn't have stdout included in Django < 1.2 
        # So, let's add it in if it's not already there.
        if not hasattr(self, 'stdout'):
            from sys import stdout
            self.stdout = stdout
 
    def handle_noargs(self, **options):
        klass_list = [fs.content_type.model_class() for fs in FieldSanitizer.objects.all()]
        object_count = _clean_class_objects(klass_list)

        self.stdout.write("\nCleaned %s Objects\n" % object_count)



"""
Management utility to list HTML elements used in Existing Content.
This could be useful when trying to decide which elements to include
in the whitelist.
"""

from django.core.management.base import NoArgsCommand
from janitor.models import FieldSanitizer, _clean_class_objects, _get_tags_used_in_content

class Command(NoArgsCommand):

    help = "Lists HTML elements used in all Models that have a related FieldSanitizer"

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)

        # BaseCommand doesn't have stdout included in Django < 1.2 
        # So, let's add it in if it's not already there.
        if not hasattr(self, 'stdout'):
            from sys import stdout
            self.stdout = stdout
 
    def handle_noargs(self, **options):
        tag_list = _get_tags_used_in_content()
        self.stdout.write("\nTags used in content:\n\n    %s\n\n" % '\n    '.join(tag_list))



from django.contrib import admin
from forms import FieldSanitizerAdminForm
from models import FieldSanitizer

class FieldSanitizerAdmin(admin.ModelAdmin):
    list_display = ('content_type', 'app_name', 'model_name', 'field_name', ) 
    form = FieldSanitizerAdminForm
admin.site.register(FieldSanitizer, FieldSanitizerAdmin)



"""
Basic tests for django-janitor. 
"""
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.management import call_command
from django.db import transaction
from django.db.models import loading
from django.test import TransactionTestCase

from janitor.models import FieldSanitizer
from janitor import whitelists

class TestJanitor(TransactionTestCase):

    def setUp(self):
        transaction.commit()
        
        if 'janitor.tests' not in settings.INSTALLED_APPS:
            settings.INSTALLED_APPS = list(settings.INSTALLED_APPS)
            settings.INSTALLED_APPS.append('janitor.tests')
            loading.cache.loaded = False
            call_command('syncdb', verbosity=1)
        from janitor.tests.models import TestModel
        self.test_model = TestModel
        self.ct = ContentType.objects.get_for_model(TestModel) 

        # some content that we'll try to clean
        self.sample_content = """<h1 id="a"><a href="/">Heading</a></h1><p style="color:blue;" class="foo">Blue <strong>text</strong><script></script></p>"""

        # Cleaned results to verify that everything's working
        self.cleaned_content = """<h1 id="a"><a href="/">Heading</a></h1><p class="foo">Blue <strong>text</strong>&lt;script&gt;&lt;/script&gt;</p>"""
        self.stripped_content = """<h1 id="a"><a href="/">Heading</a></h1><p class="foo">Blue <strong>text</strong></p>"""
    
    def tearDown(self):
        if 'janitor.tests' in settings.INSTALLED_APPS:
            from janitor.tests.models import TestModel
            TestModel.objects.all().delete()
            FieldSanitizer.objects.all().delete()
            settings.INSTALLED_APPS = list(settings.INSTALLED_APPS)
            settings.INSTALLED_APPS.remove('janitor.tests')

    def test_default_clean(self):  
        """
        Creates an instance of the test model with some sample content,
        then verifies that it gets cleaned upon saving.
        """
        fs = FieldSanitizer(content_type=self.ct, field_name="content") 
        fs.save()
        obj = self.test_model(content=self.sample_content)
        obj.save()
        self.assertEqual(obj.content, self.cleaned_content)
        self.assertEqual(fs.app_name, 'tests')
        self.assertEqual(fs.model_name, 'testmodel')
        
    def test_strip_content(self):
        """ 
        Adds an HTML comment to the class's sample content, then 
        verifies that it gets removed.
        """
        fs = FieldSanitizer(content_type=self.ct, field_name="content", 
            strip=True, strip_comments=True)
        fs.save()
        obj = self.test_model(content="<!-- Hello! -->" + self.sample_content)
        obj.save()
        self.assertEqual(obj.content, self.stripped_content)
        
    def test_default_object_data(self):
        """
        Tests that FieldSanitizer's default tags, attributes, and clean_args
        are in place.
        """
        fs = FieldSanitizer()
        self.assertEqual(fs.get_tags_list(), whitelists.basic_content_tags)
        self.assertEqual(fs.get_attributes_list(), whitelists.attributes)

        expected_args = {'styles': [], 'attributes': [u'alt', u'class', u'href', u'id', u'src', u'title'], 'strip': False, 'strip_comments': True, 'tags': [u'a', u'abbr', u'acronym', u'blockquote', u'cite', u'code', u'dd', u'del', u'dfn', u'dl', u'dt', u'em', u'h1', u'h2', u'h3', u'h4', u'h5', u'h6', u'hr', u'img', u'ins', u'kbd', u'li', u'ol', u'p', u'pre', u'q', u'samp', u'strong', u'ul']}
        self.assertEqual(fs.get_bleach_clean_args(), expected_args)



from setuptools import setup
from janitor import __version__

setup(
    name='django-janitor',
    version=__version__,
    description="django-janitor allows you to use bleach to clean HTML stored in a Model's field.",
    long_description=open('README.rst').read(),
    author='Brad Montgomery',
    author_email='brad@bradmontgomery.net',
    url='https://bitbucket.org/bkmontgomery/django-janitor/',
    license='BSD',
    packages=['janitor'],
    include_package_data=True,
    package_data = { '': ['README.rst'] },
    zip_safe=False,
    install_requires=['django', 'bleach'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Text Processing :: Markup :: HTML',
        'Topic :: Utilities',
    ]
)



"""
Management utility to clean a specific Model that is registered with a FieldSanitizer.
"""
from optparse import make_option
from django.core.management.base import CommandError, LabelCommand
from janitor.models import FieldSanitizer, _clean_class_objects, _get_tags_used_in_content

class Command(LabelCommand):

    help = "Lists HTML elements used in a Model with a related FieldSanitizer"
    args = "[appname.modelname]"
    label = 'application name.model name'

    can_import_settings = False
   
    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
       
        # BaseCommand doesn't have stdout included in Django < 1.2 
        # So, let's add it in if it's not already there.
        if not hasattr(self, 'stdout'):
            from sys import stdout
            self.stdout = stdout

    def handle_label(self, label, **options):
        try:
            app_label, model = label.lower().split('.')
        except ValueError:
            raise CommandError("Invalid app_label.model_name: %s" % label)
        
        tag_list = _get_tags_used_in_content(app_label, model)
        self.stdout.write("\nTags used in content:\n\n    %s\n\n" % '\n    '.join(tag_list))



"""
Test models.
"""
from django.db import models

class TestModel(models.Model):
    content = models.TextField()

    def __unicode__(self):
        return self.content

    class Meta:
        ordering = ['content']
        verbose_name = 'Test Model'
        verbose_name_plural = 'Test Models'

